# -*- coding: utf-8 -*-

# pycrontab.py
# This file is part of Qarte-3
#    
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2016-2017 Vincent Vande Vyvre
# Licence: GPL3
# Home page: https://launchpad.net/qarte
#
# crontab handler

import os
import pwd
import re
import tempfile
import subprocess

class PyCronTab:
    def __init__(self):
        self.user = self.get_user()
        self.cron_cmd = "/usr/bin/crontab"
        self.lines = []

    def get_user(self):
        """Return the user name.

        """
        return pwd.getpwuid(os.getuid())[0]

    def read_crontab(self):
        """Read the current crontab.

        The crontab text is transformed into a list of lines.

        Returns:
        error message if occurs else None
        """
        cmd = [self.cron_cmd, "-l", "-u", self.user]
        reply, error = subprocess.Popen(cmd, universal_newlines=True, 
                                stdout=subprocess.PIPE).communicate()

        if error is None:
            self.lines = reply.strip("\n").split("\n")

        else:
            return error

    def clean_crontab(self, indexes):
        """Remove the expired tasks from the crontab.

        Args:
        indexes -- list of indexes of pending tasks
        """
        rg = re.compile("(?<=qarte -a )(.*?)(?= >>)")
        discarded = []
        for idx, line in enumerate(self.lines):
            if not line.startswith('#'):
                match = rg.search(line)
                if match is not None:
                    index = match.group(0)
                    if index not in indexes:
                        discarded.append(idx)

        for i in reversed(discarded):
            self.lines.pop(i)

        self.write_crontab()

    def add_new_task(self, cmd, date, time):
        """Add a new line into the crontab.

        Args:
        cmd -- the command to be execute by cron
        date -- (day, month) or int(dayofweek)
        time -- (hour, minute)
        """
        timecode = self.set_time(date, time)
        self.lines.append('%s %s' %(timecode, cmd))

    def remove_task(self, task):
        """Remove a task from the crontab.

        Args:
        task -- the index of the task
        """
        txt = 'qarte -a %s' % task
        exist = False
        for idx, l in enumerate(self.lines):
            if not l.startswith('#'):
                if txt in l:
                    exist = True
                    break

        if exist:
            self.lines.pop(idx)

    def write_crontab(self):
        """Write the crontab to the system.

        """
        filed, path = tempfile.mkstemp()
        fileh = os.fdopen(filed, 'w')
        fileh.write(self.render())
        fileh.close()
        cmd = "%s %s -u %s" % (self.cron_cmd, path, self.user)
        os.system(cmd)
        os.unlink(path)

    def set_time(self, day, hour):
        """Return the time formatted.

        Args:
        day -- (day, month) or int(dayofweek)
        hour -- (hour, minute)
        """
        if isinstance(day, int):
            return "%s %s * * %s" %(hour[1], hour[0], day)

        return "%s %s %s %s *" %(hour[1], hour[0], day[0], day[1])

    def render(self):
        """Return the joined lines of the crontab.

        """
        return "\n".join(self.lines) + "\n"


